@@ -43,7 +43,7 @@
 #include <e2k-utils.h>
 
 #include <e-util/e-dialog-utils.h>
-#include <e-util/e-error.h>
+#include <e-util/e-alert-dialog.h>
 
 typedef struct {
 	const gchar *uri;
@@ -188,7 +188,6 @@ static const gchar *sd_props[] = {
 	E2K_PR_EXCHANGE_SD_BINARY,
 	E2K_PR_EXCHANGE_SD_XML
 };
-static const gint n_sd_props = sizeof (sd_props) / sizeof (sd_props[0]);
 
 /* Read the folder security descriptors and match them up with the
  * list of delegates.
@@ -214,7 +213,7 @@ get_folder_security (ExchangeDelegates *delegates)
 		return delegates->loaded_folders;
 
 	if (!exchange_account_get_global_catalog (delegates->account)) {
-		e_error_run (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegates-no-gcs-error",
+		e_alert_run_dialog_for_args (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegates-no-gcs-error",
 			     NULL);
 		return FALSE;
 	}
@@ -236,7 +235,7 @@ get_folder_security (ExchangeDelegates *delegates)
 	iter = e2k_context_bpropfind_start (
 		ctx, NULL, delegates->account->home_uri,
 		(const gchar **)hrefs->pdata, hrefs->len,
-		sd_props, n_sd_props);
+		sd_props, G_N_ELEMENTS (sd_props));
 	g_ptr_array_free (hrefs, TRUE);
 
 	while ((result = e2k_result_iter_next (iter))) {
@@ -253,14 +252,14 @@ get_folder_security (ExchangeDelegates *delegates)
 	status = e2k_result_iter_free (iter);
 
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-		e_error_run (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegates-perm-read-error",
+		e_alert_run_dialog_for_args (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegates-perm-read-error",
 			     NULL);
 		return FALSE;
 	}
 
 	if (!fill_in_sids (delegates)) {
 		delegates->loaded_folders = FALSE;
-		e_error_run (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":perm-deter-error", NULL);
+		e_alert_run_dialog_for_args (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":perm-deter-error", NULL);
 		return FALSE;
 	}
 
@@ -284,7 +283,6 @@ static const gchar *delegation_props[] = {
 	PR_DELEGATES_SEE_PRIVATE,
 	PR_CREATOR_ENTRYID
 };
-static const gint n_delegation_props = sizeof (delegation_props) / sizeof (delegation_props[0]);
 
 /* Fetch the list of delegates from the freebusy message. */
 static gboolean
@@ -302,7 +300,8 @@ get_user_list (ExchangeDelegates *delegates)
 	iter = e2k_context_bpropfind_start (ctx, NULL,
 					    delegates->account->home_uri,
 					    &exchange_localfreebusy_path, 1,
-					    delegation_props, n_delegation_props);
+					    delegation_props,
+					    G_N_ELEMENTS (delegation_props));
 	result = e2k_result_iter_next (iter);
 	if (!result || !E2K_HTTP_STATUS_IS_SUCCESSFUL (result->status)) {
 		e2k_result_iter_free (iter);
@@ -426,7 +425,7 @@ add_button_clicked_cb (GtkWidget *widget, gpointer data)
 	user = exchange_delegates_user_new_from_gc (gc, email,
 						    delegates->creator_entryid);
 	if (!user) {
-		e_error_run (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-error", email, NULL);
+		e_alert_run_dialog_for_args (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-error", email, NULL);
 		g_free (email);
 		return;
 	}
@@ -435,7 +434,7 @@ add_button_clicked_cb (GtkWidget *widget, gpointer data)
 	delegate_exchange_dn = e2k_entryid_to_dn (user->entryid);
 	if (delegate_exchange_dn && !g_ascii_strcasecmp (delegate_exchange_dn, delegates->account->legacy_exchange_dn)) {
 		g_object_unref (user);
-		e_error_run (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-own-error", NULL);
+		e_alert_run_dialog_for_args (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-own-error", NULL);
 		return;
 	}
 
@@ -443,7 +442,7 @@ add_button_clicked_cb (GtkWidget *widget, gpointer data)
 		match = delegates->users->pdata[u];
 		if (e2k_sid_binary_sid_equal (e2k_sid_get_binary_sid (user->sid),
 					      e2k_sid_get_binary_sid (match->sid))) {
-			e_error_run (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-existing",
+			e_alert_run_dialog_for_args (GTK_WINDOW (parent_window), ERROR_DOMAIN ":delegate-existing",
 				     user->display_name, NULL);
 			g_object_unref (user);
 			exchange_delegates_user_edit (delegates->account, match, parent_window);
@@ -823,7 +822,7 @@ delegates_apply (ExchangeDelegates *delegates)
 
  done:
 	if (error) {
-		e_error_run (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegate-fail-error", error, NULL);
+		e_alert_run_dialog_for_args (GTK_WINDOW (delegates->table), ERROR_DOMAIN ":delegate-fail-error", error, NULL);
 		g_free (error);
 	}
 }
@@ -961,17 +960,17 @@ exchange_delegates (ExchangeAccount *account, GtkWidget *parent)
 	add_button = gtk_button_new_from_stock ("gtk-add");
 	gtk_widget_show (add_button);
 	gtk_container_add (GTK_CONTAINER (vbuttonbox1), add_button);
-	GTK_WIDGET_SET_FLAGS (add_button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (add_button, TRUE);
 
 	edit_button = gtk_button_new_with_mnemonic (_("_Edit"));
 	gtk_widget_show (edit_button);
 	gtk_container_add (GTK_CONTAINER (vbuttonbox1), edit_button);
-	GTK_WIDGET_SET_FLAGS (edit_button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (edit_button, TRUE);
 
 	remove_button = gtk_button_new_from_stock ("gtk-remove");
 	gtk_widget_show (remove_button);
 	gtk_container_add (GTK_CONTAINER (vbuttonbox1), remove_button);
-	GTK_WIDGET_SET_FLAGS (remove_button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (remove_button, TRUE);
 
 	g_signal_connect (delegates->dialog, "response",
 			  G_CALLBACK (dialog_response), delegates);
